var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/* chinese-whisper version 0.2.11 */
(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('events')) :
        typeof define === 'function' && define.amd ? define(['exports', 'events'], factory) :
            (global = global || self, factory(global.window = {}, global.events));
}(this, function (exports, events) {
    'use strict';
    /**
     * Chinese Whispers Algorithm for Node.js
     *
     * https://github.com/huan/chinese-whispers
     * License: Apache-2.0
     * Author: Huan LI <zixia@zixia.net>
     *
     * Inspired by:
     *  - http://blog.csdn.net/liyuan123zhouhui/article/details/70312716
     *  - http://alexloveless.co.uk/data/chinese-whispers-graph-clustering-in-python/
     *  - https://github.com/uhh-lt/chinese-whispers
     *  - https://github.com/anvaka/ngraph.cw
     *
     */
    // tslint:disable:no-var-requires
    var jsnx = require('jsnetworkx');
    var knuthShuffle = require('knuth-shuffle').knuthShuffle;
    var ChineseWhispers = /** @class */ (function (_super) {
        __extends(ChineseWhispers, _super);
        function ChineseWhispers(options) {
            var _this = _super.call(this) || this;
            _this.weightFunc = options.weightFunc;
            _this.epochs = options.epochs || 15;
            _this.threshold = options.threshold;
            _this.on('change', function () { return _this.changeCounter++; });
            return _this;
        }
        ChineseWhispers.prototype.on = function (event, listener) {
            _super.prototype.on.call(this, event, listener);
            return this;
        };
        ChineseWhispers.prototype.emit = function (event) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            return _super.prototype.emit.apply(this, [event].concat(args));
        };
        ChineseWhispers.prototype.cluster = function (dataList) {
            // this.dataList = dataList
            this.graph = this.buildNetwork(dataList, this.weightFunc, this.threshold);
            // initial epoch
            this.emit('epoch', this.graph, -1);
            // run Chinese Whispers
            // I default to 10 iterations. This number is usually low.
            // After a certain number (individual to the data set) no further clustering occurs
            var epochs = this.epochs;
            while (epochs--) {
                this.changeCounter = 0;
                this.iterate();
                this.emit('epoch', this.graph, this.changeCounter);
            }
            var clusterList = this.buildClusterList(this.graph);
            return clusterList;
        };
        ChineseWhispers.prototype.iterate = function () {
            var nodeList = this.graph.nodes();
            // I randomize the nodes to give me an arbitrary start point
            knuthShuffle(nodeList); // orignal array modified
            for (var _i = 0, nodeList_1 = nodeList; _i < nodeList_1.length; _i++) {
                var node = nodeList_1[_i];
                this.relabelNode(node);
            }
        };
        ChineseWhispers.prototype.relabelNode = function (node) {
            var newLabel = this.recalcLabel(node);
            var nodeAttr = this.graph.node.get(node);
            if (nodeAttr.label !== newLabel) {
                // set the label of target node to the winning local label
                this.emit('change', node, nodeAttr.label, newLabel);
                nodeAttr.label = newLabel;
                // console.log('set node ' + this.data[node] + ' label to ' + this.data[parseInt(newLabel)])
            }
        };
        ChineseWhispers.prototype.recalcLabel = function (node) {
            var nxNode = this.graph.get(node);
            var neighborList = this.graph.neighbors(node);
            if (neighborList.length === 0) {
                var attr = this.graph.node.get(node);
                return attr.label; // return self label if no neighbors
            }
            // console.log('neighbors of node ' + this.data[node]
            //     + ' is: ', neighborList.map((i: number) => this.data[i]))
            var labelWeightMap = {};
            // do an inventory of the given nodes neighbours and edge weights
            for (var _i = 0, neighborList_1 = neighborList; _i < neighborList_1.length; _i++) {
                var neighbor = neighborList_1[_i];
                var neighborAttr = this.graph.node.get(neighbor);
                var label = neighborAttr.label;
                if (!(label in labelWeightMap)) {
                    labelWeightMap[label] = 0;
                }
                var edgeAttr = nxNode.get(neighbor);
                var weight = edgeAttr.weight;
                // console.log('### ', weight, ' ###')
                labelWeightMap[label] += weight;
            }
            // find the label with the highest edge weight sum
            var max = 0;
            // In javascript the keys of object can only be strings
            //  - https://stackoverflow.com/a/41870625/1123955
            var maxLabel = '-1';
            // for (const label in labelWeightMap) {
            //   console.log('labelWeight node: ' + this.data[node] + ' - '
            //       + this.data[parseInt(label)] + ' is: ', labelWeightMap[label])
            // }
            for (var label in labelWeightMap) {
                if (labelWeightMap[label] > max) {
                    max = labelWeightMap[label];
                    maxLabel = label;
                }
            }
            return maxLabel;
        };
        ChineseWhispers.prototype.buildClusterList = function (G) {
            var clusterList = [];
            var labelIndexMap = {};
            var index = 0;
            for (var _i = 0, _a = G.nodes(); _i < _a.length; _i++) {
                var node = _a[_i];
                var label = G.node.get(node).label;
                if (!(label in labelIndexMap)) {
                    labelIndexMap[label] = index++;
                }
                var labelIndex = labelIndexMap[label];
                if (!(labelIndex in clusterList)) {
                    clusterList[labelIndex] = [];
                }
                clusterList[labelIndex].push(node);
            }
            return clusterList;
        };
        ChineseWhispers.prototype.buildNetwork = function (dataList, weightFunc, threshold) {
            var nodeList = Object.keys(dataList).map(function (k) { return parseInt(k, 10); }); // [0, 1, 2, ..., data.length - 1]
            // const nodeList: CWNode[] = [...data.keys()] // [0, 1, 2, ..., data.length - 1]
            var edgeList = [];
            for (var i = 0; i < dataList.length; i++) {
                for (var j = i + 1; j < dataList.length; j++) {
                    var weight = weightFunc(dataList[i], dataList[j]);
                    if (threshold && threshold > weight) {
                        // console.log('threshold: ', threshold, ' weight: ', weight)
                        // skip this edge because it's weight is below threshold
                        this.emit('edge');
                    }
                    else {
                        var edge = [i, j, { weight: weight }];
                        edgeList.push(edge);
                        this.emit('edge', edge);
                    }
                }
            }
            // initialize the graph
            var G = new jsnx.Graph();
            // Add nodes
            G.addNodesFrom(nodeList);
            // CW needs an arbitrary, unique label for each node before initialisation
            // Here I use the ID of the node since I know it's unique
            // You could use a random number or a counter or anything really
            for (var _i = 0, _a = G.nodes(); _i < _a.length; _i++) {
                var n = _a[_i];
                var nodeAttr = G.node.get(n);
                nodeAttr.label = n;
            }
            // add edges
            G.addEdgesFrom(edgeList);
            // console.log(edgeList)
            return G;
        };
        return ChineseWhispers;
    }(events.EventEmitter));
    exports.ChineseWhispers = ChineseWhispers;
    exports.default = ChineseWhispers;
    Object.defineProperty(exports, '__esModule', { value: true });
}));
/* https://git.io/zixia/ */
//# sourceMappingURL=chinese-whispers.es6.umd.js.map
