/// <reference types="node" />
/**
 * Chinese Whispers Algorithm for Node.js
 *
 * https://github.com/huan/chinese-whispers
 * License: Apache-2.0
 * Author: Huan LI <zixia@zixia.net>
 *
 * Inspired by:
 *  - http://blog.csdn.net/liyuan123zhouhui/article/details/70312716
 *  - http://alexloveless.co.uk/data/chinese-whispers-graph-clustering-in-python/
 *  - https://github.com/uhh-lt/chinese-whispers
 *  - https://github.com/anvaka/ngraph.cw
 *
 */
import { EventEmitter } from 'events';
export declare type WeightFunc<T> = (a: T, b: T) => number;
export declare type CWNode = number;
export declare type CWEdge = [number, number, {
    weight: number;
}];
export declare type Cluster = number[];
export interface ChineseWhispersOptions<T> {
    weightFunc: WeightFunc<T>;
    epochs?: number;
    threshold?: number;
}
export declare type CWEventName = 'edge' | 'epoch' | 'change';
export declare class ChineseWhispers<T> extends EventEmitter {
    private weightFunc;
    private epochs;
    private threshold?;
    private graph;
    private changeCounter;
    constructor(options: ChineseWhispersOptions<T>);
    on(event: 'edge', listener: (edge: CWEdge) => void): this;
    on(event: 'change', listener: (node: CWNode, oldLabel: string, newLabel: string) => void): this;
    on(event: 'epoch', listener: (graph: any, changeCounter: number) => void): this;
    on(event: never, listener: any): this;
    emit(event: 'edge', edge?: CWEdge): boolean;
    emit(event: 'change', node: CWNode, oldLabel: string, newLabel: string): boolean;
    emit(event: 'epoch', graph: any, changeCounter: number): boolean;
    emit(event: never, ...args: any[]): boolean;
    cluster(dataList: T[]): Cluster[];
    iterate(): void;
    relabelNode(node: CWNode): void;
    recalcLabel(node: CWNode): string;
    buildClusterList(G: any): Cluster[];
    buildNetwork(dataList: T[], weightFunc: WeightFunc<T>, threshold?: number): any;
}
export default ChineseWhispers;
